@@ -19,7 +19,9 @@ package org.uberfire.ext.editor.commons.client;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.HashSet;
+import java.util.Optional;
 import java.util.Set;
+import java.util.function.Supplier;
 import javax.enterprise.event.Event;
 import javax.enterprise.event.Observes;
 import javax.inject.Inject;
@@ -28,6 +30,7 @@ import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.user.client.ui.IsWidget;
 import org.jboss.errai.common.client.api.Caller;
 import org.jboss.errai.common.client.api.RemoteCallback;
+import org.jboss.errai.ioc.client.api.ManagedInstance;
 import org.uberfire.backend.vfs.ObservablePath;
 import org.uberfire.backend.vfs.Path;
 import org.uberfire.client.mvp.PlaceManager;
@@ -40,12 +43,14 @@ import org.uberfire.ext.editor.commons.client.event.ConcurrentRenameIgnoredEvent
 import org.uberfire.ext.editor.commons.client.history.VersionRecordManager;
 import org.uberfire.ext.editor.commons.client.menu.BasicFileMenuBuilder;
 import org.uberfire.ext.editor.commons.client.menu.MenuItems;
+import org.uberfire.ext.editor.commons.client.menu.common.SaveAndRenameCommandBuilder;
 import org.uberfire.ext.editor.commons.client.resources.i18n.CommonConstants;
 import org.uberfire.ext.editor.commons.client.validation.DefaultFileNameValidator;
 import org.uberfire.ext.editor.commons.client.validation.Validator;
 import org.uberfire.ext.editor.commons.service.support.SupportsCopy;
 import org.uberfire.ext.editor.commons.service.support.SupportsDelete;
 import org.uberfire.ext.editor.commons.service.support.SupportsRename;
+import org.uberfire.ext.editor.commons.service.support.SupportsSaveAndRename;
 import org.uberfire.ext.editor.commons.version.events.RestoreEvent;
 import org.uberfire.java.nio.base.version.VersionRecord;
 import org.uberfire.mvp.Command;
@@ -64,7 +69,7 @@ import static org.uberfire.ext.widgets.common.client.common.ConcurrentChangePopu
 import static org.uberfire.ext.widgets.common.client.common.ConcurrentChangePopup.newConcurrentRename;
 import static org.uberfire.ext.widgets.common.client.common.ConcurrentChangePopup.newConcurrentUpdate;
 
-public abstract class BaseEditor {
+public abstract class BaseEditor<T, M> {
 
     protected boolean isReadOnly;
 
@@ -86,12 +91,18 @@ public abstract class BaseEditor {
     @Inject
     protected VersionRecordManager versionRecordManager;
 
+    @Inject
+    protected ManagedInstance<BasicFileMenuBuilder> menuBuilderManagedInstance;
+
     @Inject
     protected BasicFileMenuBuilder menuBuilder;
 
     @Inject
     protected DefaultFileNameValidator fileNameValidator;
 
+    @Inject
+    protected SaveAndRenameCommandBuilder<T, M> saveAndRenameCommandBuilder;
+
     @Inject
     protected Event<ConcurrentDeleteAcceptedEvent> concurrentDeleteAcceptedEvent;
 
@@ -104,11 +115,12 @@ public abstract class BaseEditor {
     @Inject
     protected Event<ConcurrentRenameIgnoredEvent> concurrentRenameIgnoredEvent;
 
-    protected Set<MenuItems> menuItems = new HashSet<MenuItems>();
+    protected Set<MenuItems> menuItems = new HashSet<>();
 
     protected PlaceRequest place;
     protected ClientResourceType type;
     protected Integer originalHash;
+    protected Integer metadataOriginalHash;
     private boolean displayShowMoreVersions;
 
     //for test purposes only
@@ -210,7 +222,7 @@ public abstract class BaseEditor {
      */
     protected void makeMenuBar() {
         if (menuItems.contains(SAVE)) {
-            menuBuilder.addSave(this::onSave);
+            menuBuilder.addSave(getOnSave());
         }
 
         if (menuItems.contains(COPY)) {
@@ -219,9 +231,7 @@ public abstract class BaseEditor {
                                 getCopyServiceCaller());
         }
         if (menuItems.contains(RENAME)) {
-            menuBuilder.addRename(versionRecordManager.getPathToLatest(),
-                                  getRenameValidator(),
-                                  getRenameServiceCaller());
+            menuBuilder.addRename(getSaveAndRename());
         }
         if (menuItems.contains(DELETE)) {
             menuBuilder.addDelete(versionRecordManager.getCurrentPath(),
@@ -235,6 +245,70 @@ public abstract class BaseEditor {
         }
     }
 
+    Command getOnSave() {
+        return this::onSave;
+    }
+
+    protected Command getSaveAndRename() {
+
+        return getSaveAndRenameCommandBuilder()
+                .addPathSupplier(getPathSupplier())
+                .addValidator(getRenameValidator())
+                .addValidator(getSaveValidator())
+                .addRenameService(getSaveAndRenameServiceCaller())
+                .addMetadataSupplier(getMetadataSupplier())
+                .addContentSupplier(getContentSupplier())
+                .addIsDirtySupplier(isDirtySupplier())
+                .addSuccessCallback(onSuccess())
+                .build();
+    }
+
+    Supplier<Boolean> getSaveValidator() {
+
+        return () -> {
+
+            if (isReadOnly && versionRecordManager.isCurrentLatest()) {
+                baseView.alertReadOnly();
+                return false;
+            } else if (isReadOnly && !versionRecordManager.isCurrentLatest()) {
+                versionRecordManager.restoreToCurrentVersion();
+                return false;
+            }
+
+            if (concurrentUpdateSessionInfo != null) {
+                showConcurrentUpdatePopup();
+                return false;
+            } else {
+                return true;
+            }
+        };
+    }
+
+    protected ParameterizedCommand<Path> onSuccess() {
+
+        return (path) -> {
+
+            final T content = getContentSupplier().get();
+            final M metadata = getMetadataSupplier().get();
+
+            setOriginalHash(content.hashCode());
+
+            Optional<M> optionalMetadata = Optional.ofNullable(getMetadataSupplier().get());
+
+            if (optionalMetadata.isPresent()) {
+                setMetadataOriginalHash(metadata.hashCode());
+            }
+        };
+    }
+
+    protected SaveAndRenameCommandBuilder<T, M> getSaveAndRenameCommandBuilder() {
+        return saveAndRenameCommandBuilder;
+    }
+
+    protected Supplier<Path> getPathSupplier() {
+        return () -> versionRecordManager.getPathToLatest();
+    }
+
     /**
      * If you want to customize the menu construction override this method. {@link BaseEditor#makeMenuBar()}
      * should be used to add items to the {@link BasicFileMenuBuilder}. This method then instructs
@@ -384,17 +458,9 @@ public abstract class BaseEditor {
 
     protected void onSave() {
 
-        if (isReadOnly && versionRecordManager.isCurrentLatest()) {
-            baseView.alertReadOnly();
-            return;
-        } else if (isReadOnly && !versionRecordManager.isCurrentLatest()) {
-            versionRecordManager.restoreToCurrentVersion();
-            return;
-        }
+        final boolean isValid = getSaveValidator().get();
 
-        if (concurrentUpdateSessionInfo != null) {
-            showConcurrentUpdatePopup();
-        } else {
+        if (isValid) {
             save();
         }
     }
@@ -423,7 +489,7 @@ public abstract class BaseEditor {
         ).show();
     }
 
-    protected RemoteCallback<Path> getSaveSuccessCallback(final int newHash) {
+    public RemoteCallback<Path> getSaveSuccessCallback(final int newHash) {
         return new RemoteCallback<Path>() {
             @Override
             public void callback(final Path path) {
@@ -488,6 +554,47 @@ public abstract class BaseEditor {
 
     protected abstract void loadContent();
 
+    protected Supplier<T> getContentSupplier() {
+        return () -> null;
+    }
+
+    protected Supplier<Boolean> isDirtySupplier() {
+        return () -> isContentDirty() || isMetadataDirty();
+    }
+
+    boolean isMetadataDirty() {
+
+        final Optional<M> optionalMetadata = Optional.ofNullable(getMetadataSupplier().get());
+
+        if (optionalMetadata.isPresent()) {
+            return isMetadataDirty(optionalMetadata.get());
+        } else {
+            return false;
+        }
+    }
+
+    private boolean isMetadataDirty(final M metadata) {
+        final Integer metadataCurrentHash = metadata.hashCode();
+        return !metadataCurrentHash.equals(metadataOriginalHash);
+    }
+
+    boolean isContentDirty() {
+
+        final T currentContent;
+
+        try {
+            currentContent = getContentSupplier().get();
+        } catch (final Exception e) {
+            return false;
+        }
+
+        return isDirty(currentContent.hashCode());
+    }
+
+    protected Supplier<M> getMetadataSupplier() {
+        return () -> null;
+    }
+
     /**
      * Needs to be overwritten for save to work
      */
@@ -503,6 +610,10 @@ public abstract class BaseEditor {
         return null;
     }
 
+    protected Caller<? extends SupportsSaveAndRename<T, M>> getSaveAndRenameServiceCaller() {
+        return null;
+    }
+
     protected Caller<? extends SupportsCopy> getCopyServiceCaller() {
         return null;
     }
@@ -526,5 +637,8 @@ public abstract class BaseEditor {
     public VersionRecordManager getVersionRecordManager() {
         return this.versionRecordManager;
     }
-}
 
+    public void setMetadataOriginalHash(final Integer metadataOriginalHash) {
+        this.metadataOriginalHash = metadataOriginalHash;
+    }
+}
