@@ -16,6 +16,8 @@
 
 package org.uberfire.ext.layout.editor.client.components.columns;
 
+import java.util.List;
+import java.util.function.Supplier;
 import javax.enterprise.context.Dependent;
 import javax.enterprise.event.Observes;
 import javax.inject.Inject;
@@ -32,12 +34,15 @@ import org.jboss.errai.ui.shared.api.annotations.Templated;
 import org.uberfire.client.mvp.UberElement;
 import org.uberfire.client.workbench.docks.UberfireDocksInteractionEvent;
 import org.uberfire.ext.layout.editor.api.editor.LayoutComponent;
+import org.uberfire.ext.layout.editor.api.editor.LayoutTemplate;
 import org.uberfire.ext.layout.editor.client.api.RenderingContext;
 import org.uberfire.ext.layout.editor.client.infra.ColumnDrop;
 import org.uberfire.ext.layout.editor.client.infra.ContainerResizeEvent;
 import org.uberfire.ext.layout.editor.client.infra.DragComponentEndEvent;
 import org.uberfire.ext.layout.editor.client.infra.DragHelperComponentColumn;
+import org.uberfire.ext.layout.editor.client.resources.i18n.CommonConstants;
 import org.uberfire.ext.layout.editor.client.widgets.KebabWidget;
+import org.uberfire.ext.properties.editor.model.PropertyEditorCategory;
 import org.uberfire.mvp.Command;
 
 import static org.jboss.errai.common.client.dom.DOMUtil.addCSSClass;
@@ -97,8 +102,15 @@ public class ComponentColumnView
         this.presenter = presenter;
     }
 
+    @Override
+    public List<PropertyEditorCategory> getPropertyCategories() {
+        return helper.getLayoutDragComponentProperties();
+    }
+
     @Override
     public void setupWidget() {
+        String txt = presenter.isSelectable() ? CommonConstants.INSTANCE.SelectColumnHint() : CommonConstants.INSTANCE.DragColumnHint();
+        content.setTitle(txt);
         setupEvents();
         setupKebabWidget();
         setupOnResize();
@@ -340,16 +352,19 @@ public class ComponentColumnView
             removeCSSClass(colDown,
                            "componentDropInColumnPreview");
         });
+        content.setOnclick(e -> {
+            e.preventDefault();
+            presenter.onSelected();
+        });
         content.setOnmouseout(e -> {
             removeCSSClass(content,
-                           "componentMovePreview");
+                    "componentMovePreview");
         });
         content.setOnmouseover(e -> {
             e.preventDefault();
             addCSSClass(content,
                         "componentMovePreview");
         });
-
         content.setOndragend(e -> {
             e.stopPropagation();
             removeCSSClass(row,
@@ -518,9 +533,11 @@ public class ComponentColumnView
 
     @Override
     public void showConfigComponentModal(Command configurationFinish,
-                                         Command configurationCanceled) {
+                                         Command configurationCanceled,
+                                         Supplier<LayoutTemplate> currentLayoutTemplateSupplier) {
         helper.showConfigModal(configurationFinish,
-                               configurationCanceled);
+                               configurationCanceled,
+                               currentLayoutTemplateSupplier);
     }
 
     @Override
@@ -529,8 +546,20 @@ public class ComponentColumnView
     }
 
     @Override
-    public void setup(LayoutComponent layoutComponent) {
-        helper.setLayoutComponent(layoutComponent);
+    public void setup(LayoutComponent layoutComponent,
+                      LayoutTemplate.Style pageStyle) {
+        helper.setup(layoutComponent, pageStyle);
+    }
+
+    @Override
+    public void setSelected(boolean selected) {
+        removeCSSClass(content, "componentMovePreview");
+        String txt = presenter.isSelectable() ? CommonConstants.INSTANCE.SelectColumnHint() : CommonConstants.INSTANCE.DragColumnHint();
+        content.setTitle(txt);
+        if (selected) {
+            addCSSClass(content, "componentMovePreview");
+            content.setTitle(CommonConstants.INSTANCE.UnselectColumnHint());
+        }
     }
 
     private HTMLElement getPreviewElement() {
