@@ -20,6 +20,9 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
+import java.util.function.Supplier;
+
 import javax.annotation.PostConstruct;
 import javax.annotation.PreDestroy;
 import javax.enterprise.context.Dependent;
@@ -29,24 +32,27 @@ import javax.enterprise.inject.Instance;
 import javax.inject.Inject;
 
 import org.uberfire.client.mvp.UberElement;
-import org.uberfire.ext.layout.editor.api.editor.LayoutComponent;
+import org.uberfire.ext.layout.editor.api.css.CssProperty;
+import org.uberfire.ext.layout.editor.api.css.CssValue;
 import org.uberfire.ext.layout.editor.api.editor.LayoutRow;
 import org.uberfire.ext.layout.editor.api.editor.LayoutTemplate;
 import org.uberfire.ext.layout.editor.client.api.ComponentDropEvent;
+import org.uberfire.ext.layout.editor.client.api.ComponentDropType;
+import org.uberfire.ext.layout.editor.client.api.LayoutEditorElement;
+import org.uberfire.ext.layout.editor.client.api.LayoutEditorElementType;
 import org.uberfire.ext.layout.editor.client.components.columns.Column;
 import org.uberfire.ext.layout.editor.client.components.rows.EmptyDropRow;
 import org.uberfire.ext.layout.editor.client.components.rows.Row;
 import org.uberfire.ext.layout.editor.client.components.rows.RowDnDEvent;
 import org.uberfire.ext.layout.editor.client.components.rows.RowDrop;
-import org.uberfire.ext.layout.editor.client.infra.BeanHelper;
-import org.uberfire.ext.layout.editor.client.infra.ColumnDrop;
-import org.uberfire.ext.layout.editor.client.infra.LayoutTemplateAdapter;
-import org.uberfire.ext.layout.editor.client.infra.RowResizeEvent;
-import org.uberfire.ext.layout.editor.client.infra.UniqueIDGenerator;
+import org.uberfire.ext.layout.editor.client.event.LayoutEditorElementSelectEvent;
+import org.uberfire.ext.layout.editor.client.event.LayoutEditorElementUnselectEvent;
+import org.uberfire.ext.layout.editor.client.infra.*;
+import org.uberfire.ext.properties.editor.model.PropertyEditorCategory;
 import org.uberfire.mvp.ParameterizedCommand;
 
 @Dependent
-public class Container {
+public class Container implements LayoutEditorElement {
 
     private final Instance<Row> rowInstance;
     private final Instance<EmptyDropRow> emptyDropRowInstance;
@@ -54,6 +60,7 @@ public class Container {
     private LayoutTemplate layoutTemplate;
     private String id;
     private UniqueIDGenerator idGenerator = new UniqueIDGenerator();
+    private LayoutEditorCssHelper layoutCssHelper;
     private List<Row> rows = new ArrayList<>();
     private EmptyDropRow emptyDropRow;
     private String layoutName;
@@ -62,16 +69,26 @@ public class Container {
     private Map<String, String> properties = new HashMap<>();
     private Event<ComponentDropEvent> componentDropEvent;
     private LayoutTemplate.Style pageStyle = LayoutTemplate.Style.FLUID;
+    private Event<LayoutEditorElementSelectEvent> containerSelectEvent;
+    private Event<LayoutEditorElementUnselectEvent> containerUnselectEvent;
+    private boolean selectable = false;
+    private boolean selected = false;
 
     @Inject
     public Container(final View view,
+                     LayoutEditorCssHelper layoutCssHelper,
                      Instance<Row> rowInstance,
                      Instance<EmptyDropRow> emptyDropRowInstance,
-                     Event<ComponentDropEvent> componentDropEvent) {
+                     Event<ComponentDropEvent> componentDropEvent,
+                     Event<LayoutEditorElementSelectEvent> containerSelectEvent,
+                     Event<LayoutEditorElementUnselectEvent> containerUnselectEvent) {
+        this.layoutCssHelper = layoutCssHelper;
         this.rowInstance = rowInstance;
         this.emptyDropRowInstance = emptyDropRowInstance;
         this.view = view;
         this.componentDropEvent = componentDropEvent;
+        this.containerSelectEvent = containerSelectEvent;
+        this.containerUnselectEvent = containerUnselectEvent;
         this.id = idGenerator.createContainerID();
     }
 
@@ -97,6 +114,49 @@ public class Container {
         rows = new ArrayList<>();
     }
 
+    @Override
+    public String getId() {
+        return id;
+    }
+
+    @Override
+    public LayoutEditorElementType geElementType() {
+        return LayoutEditorElementType.CONTAINER;
+    }
+
+    @Override
+    public LayoutEditorElement getParentElement() {
+        return null;
+    }
+
+    @Override
+    public void setSelectable(boolean selectable) {
+        this.selectable = selectable;
+        view.setSelectEnabled(selectable);
+    }
+
+    public boolean isSelectable() {
+        return selectable;
+    }
+
+    @Override
+    public boolean isSelected() {
+        return selected;
+    }
+
+    @Override
+    public void setSelected(boolean status) {
+        if (isSelectable()) {
+            if (status) {
+                selected = true;
+                view.setSelected(true);
+            } else {
+                selected = false;
+                view.setSelected(false);
+            }
+        }
+    }
+
     private void createEmptyDropRow() {
         emptyDropRow = createEmptyRow();
         view.addEmptyRow(emptyDropRow.getView());
@@ -110,6 +170,7 @@ public class Container {
         this.pageStyle = pageStyle;
         this.emptyTitleText = emptyTitleText;
         this.emptySubTitleText = emptySubTitleText;
+        clearView();
         createEmptyDropRow();
         setupResizeRows();
     }
@@ -118,20 +179,24 @@ public class Container {
                      String emptyTitleText,
                      String emptySubTitleText) {
         this.layoutTemplate = layoutTemplate;
+        this.layoutName = layoutTemplate.getName();
         this.pageStyle = layoutTemplate.getStyle();
         this.emptyTitleText = emptyTitleText;
         this.emptySubTitleText = emptySubTitleText;
+        this.rows.forEach(this::destroy);
+        this.rows.clear();
         if (!layoutTemplate.isEmpty()) {
-            this.layoutName = layoutTemplate.getName();
             this.properties = layoutTemplate.getLayoutProperties();
             for (LayoutRow layoutRow : layoutTemplate.getRows()) {
                 rows.add(load(layoutRow));
             }
             updateView();
         } else {
+            clearView();
             createEmptyDropRow();
         }
         setupResizeRows();
+        setupCssProperties();
     }
 
     public void reset() {
@@ -140,7 +205,7 @@ public class Container {
         emptyTitleText = null;
         emptySubTitleText = null;
         layoutName = null;
-        properties = null;
+        properties.clear();;
         emptyDropRow = null;
         pageStyle = LayoutTemplate.Style.FLUID;
     }
@@ -162,15 +227,16 @@ public class Container {
     public ParameterizedCommand<RowDrop> createEmptyDropCommand() {
         return (drop) -> {
             destroy(emptyDropRow);
-            notifyDrop(drop.getComponent());
+            notifyDrop(drop);
             rows.add(createRow(drop,
                                Row.ROW_DEFAULT_HEIGHT));
             updateView();
         };
     }
 
-    private void notifyDrop(LayoutComponent component) {
-        componentDropEvent.fire(new ComponentDropEvent(component));
+    private void notifyDrop(RowDrop rowDrop) {
+        componentDropEvent.fire(new ComponentDropEvent(rowDrop.getComponent(),
+                                                       rowDrop.getType().equals(ComponentDropType.FROM_MOVE)));
     }
 
     private Row createRow(RowDrop drop,
@@ -179,6 +245,7 @@ public class Container {
         row.init(createRowDropCommand(),
                  createRemoveRowCommand(),
                  createRemoveComponentCommand(),
+                 createCurrentLayoutTemplateSupplier(),
                  height);
         row.withOneColumn(drop.getComponent(),
                           drop.newComponent());
@@ -186,6 +253,10 @@ public class Container {
         return row;
     }
 
+    Supplier<LayoutTemplate> createCurrentLayoutTemplateSupplier() {
+        return () -> toLayoutTemplate();
+    }
+
     private ParameterizedCommand<Row> createRemoveRowCommand() {
         return (row) -> {
             removeRow(row);
@@ -276,9 +347,15 @@ public class Container {
     }
 
     private void removeOldComponent(Column column) {
-        Row rowToRemove = null;
-        for (Row row : rows) {
-            row.removeChildColumn(column);
+
+        // Search the row that contains the column
+        Optional<Row> rowOptional = rows.stream()
+                .filter(row -> row.cointainsColumn(column))
+                .findAny();
+
+        // If the row is present remove it!
+        if (rowOptional.isPresent()) {
+            rowOptional.get().removeChildColumn(column);
         }
     }
 
@@ -310,7 +387,7 @@ public class Container {
                 setupRowSize(currentRow);
             }
         }
-        notifyDrop(dropRow.getComponent());
+        notifyDrop(dropRow);
     }
 
     private void setupRowSize(Row currentRow) {
@@ -383,13 +460,15 @@ public class Container {
         row.load(createRowDropCommand(),
                  layoutRow,
                  createRemoveRowCommand(),
-                 createRemoveComponentCommand());
+                 createRemoveComponentCommand(),
+                 createCurrentLayoutTemplateSupplier());
         return row;
     }
 
     protected Row createInstanceRow() {
         Row row = rowInstance.get();
-        row.setup(idGenerator.createRowID(id),
+        row.setSelectable(selectable);
+        row.setup(this, idGenerator.createRowID(id),
                   pageStyle);
         return row;
     }
@@ -404,10 +483,34 @@ public class Container {
         return properties.get(key);
     }
 
+    @Override
     public Map<String, String> getProperties() {
         return properties;
     }
 
+    @Override
+    public void setProperty(String property, String value) {
+        properties.put(property, value);
+        setupCssProperties();
+    }
+
+    @Override
+    public void removeProperty(String property) {
+        properties.remove(property);
+        setupCssProperties();
+    }
+
+    @Override
+    public void clearProperties() {
+        properties.clear();
+        setupCssProperties();
+    }
+
+    @Override
+    public List<PropertyEditorCategory> getPropertyCategories() {
+        return layoutCssHelper.getContainerPropertyCategories(this);
+    }
+
     public LayoutTemplate toLayoutTemplate() {
         LayoutTemplate convert = LayoutTemplateAdapter.convert(this);
         return convert;
@@ -476,7 +579,7 @@ public class Container {
         if (hasDownSibling(index,
                            rows)) {
             Row downSibling = rows.get(index + 1);
-            return downSibling.getHeight() > 2;
+            return downSibling.getHeight() > 1;
         }
         return false;
     }
@@ -488,7 +591,7 @@ public class Container {
 
     private boolean canResizeUp(int index,
                                 List<Row> rows) {
-        return (rows.get(index - 1).getHeight() > 2);
+        return (rows.get(index - 1).getHeight() > 1);
     }
 
     private boolean firstRow(int index) {
@@ -501,7 +604,12 @@ public class Container {
         }
     }
 
-    public UberElement<Container> getView() {
+    private void setupCssProperties() {
+        List<CssValue> cssValueList = layoutCssHelper.readCssValues(properties);
+        view.applyCssValues(cssValueList);
+    }
+
+    public View getView() {
         updateView();
         return view;
     }
@@ -558,6 +666,20 @@ public class Container {
         return pageStyle;
     }
 
+    public void onSelected() {
+        if (isSelectable()) {
+            if (selected) {
+                containerUnselectEvent.fire(new LayoutEditorElementUnselectEvent(this));
+            } else {
+                containerSelectEvent.fire(new LayoutEditorElementSelectEvent(this));
+            }
+        }
+    }
+
+    public List<Row> getChildElements() {
+        return rows;
+    }
+
     public interface View extends UberElement<Container> {
 
         void addRow(UberElement<Row> view);
@@ -567,5 +689,11 @@ public class Container {
         void addEmptyRow(UberElement<EmptyDropRow> emptyDropRow);
 
         void pageMode();
+
+        void setSelectEnabled(boolean enabled);
+
+        void setSelected(boolean selected);
+
+        void applyCssValues(List<CssValue> cssValues);
     }
 }
