@@ -16,6 +16,10 @@
 
 package org.uberfire.ext.metadata.io;
 
+import static org.kie.soup.commons.validation.PortablePreconditions.checkNotNull;
+import static org.uberfire.java.nio.file.StandardWatchEventKind.ENTRY_CREATE;
+import static org.uberfire.java.nio.file.StandardWatchEventKind.ENTRY_MODIFY;
+
 import java.net.URI;
 import java.util.ArrayList;
 import java.util.HashSet;
@@ -23,6 +27,8 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.ExecutorService;
+import java.util.stream.Stream;
+import java.util.stream.StreamSupport;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -41,6 +47,7 @@ import org.uberfire.java.nio.base.WatchContext;
 import org.uberfire.java.nio.base.dotfiles.DotFileUtils;
 import org.uberfire.java.nio.file.DeleteOption;
 import org.uberfire.java.nio.file.DirectoryNotEmptyException;
+import org.uberfire.java.nio.file.DirectoryStream;
 import org.uberfire.java.nio.file.FileSystem;
 import org.uberfire.java.nio.file.FileSystemAlreadyExistsException;
 import org.uberfire.java.nio.file.FileSystemNotFoundException;
@@ -54,22 +61,17 @@ import org.uberfire.java.nio.file.WatchService;
 import org.uberfire.java.nio.file.attribute.FileAttribute;
 import org.uberfire.java.nio.file.attribute.FileAttributeView;
 
-import static org.kie.soup.commons.validation.PortablePreconditions.checkNotNull;
-import static org.uberfire.java.nio.file.StandardWatchEventKind.ENTRY_CREATE;
-import static org.uberfire.java.nio.file.StandardWatchEventKind.ENTRY_MODIFY;
-
 public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
 
     private static final Logger LOGGER = LoggerFactory.getLogger(IOServiceIndexedImpl.class);
 
     private final MetaIndexEngine indexEngine;
-    private final BatchIndex batchIndex;
 
     private final Class<? extends FileAttributeView>[] views;
     private final List<String> watchedList = new ArrayList<>();
-    private final List<WatchService> watchServices = new ArrayList<WatchService>();
+    private final List<WatchService> watchServices = new ArrayList<>();
+    private final BatchIndex batchIndex;
 
-    private final Observer observer;
     private ExecutorService executorService;
 
     public IOServiceIndexedImpl(final MetaIndexEngine indexEngine,
@@ -123,16 +125,10 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
         super();
         this.indexEngine = checkNotNull("indexEngine",
                                         indexEngine);
-        this.observer = checkNotNull("observer",
-                                     observer);
-        this.batchIndex = new BatchIndex(indexEngine,
-                                         this,
-                                         observer,
-                                         executorService,
-                                         views);
         this.views = views;
 
         this.executorService = executorService;
+        this.batchIndex = new BatchIndex(indexEngine, this, observer, executorService, views);
     }
 
     public IOServiceIndexedImpl(final String id,
@@ -143,15 +139,9 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
         super(id);
         this.indexEngine = checkNotNull("indexEngine",
                                         indexEngine);
-        this.observer = checkNotNull("observer",
-                                     observer);
-        this.batchIndex = new BatchIndex(indexEngine,
-                                         this,
-                                         observer,
-                                         executorService,
-                                         views);
         this.views = views;
         this.executorService = executorService;
+        this.batchIndex = new BatchIndex(indexEngine, this, observer, executorService, views);
     }
 
     public IOServiceIndexedImpl(final IOWatchService watchService,
@@ -162,16 +152,10 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
         super(watchService);
         this.indexEngine = checkNotNull("indexEngine",
                                         indexEngine);
-        this.observer = checkNotNull("observer",
-                                     observer);
-        this.batchIndex = new BatchIndex(indexEngine,
-                                         this,
-                                         observer,
-                                         executorService,
-                                         views);
         this.views = views;
 
         this.executorService = executorService;
+        this.batchIndex = new BatchIndex(indexEngine, this, observer, executorService, views);
     }
 
     public IOServiceIndexedImpl(final String id,
@@ -184,16 +168,10 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
               watchService);
         this.indexEngine = checkNotNull("indexEngine",
                                         indexEngine);
-        this.observer = checkNotNull("observer",
-                                     observer);
-        this.batchIndex = new BatchIndex(indexEngine,
-                                         this,
-                                         observer,
-                                         executorService,
-                                         views);
         this.views = views;
 
         this.executorService = executorService;
+        this.batchIndex = new BatchIndex(indexEngine, this, observer, executorService, views);
     }
 
     @Override
@@ -201,7 +179,9 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
             throws IllegalArgumentException, FileSystemNotFoundException,
             ProviderNotFoundException, SecurityException {
         final FileSystem fs = super.getFileSystem(uri);
-        indexIfFresh(fs);
+        if (shouldPerformInitialIndex(fs)) {
+            setupBatchIndex(fs);
+        }
         setupWatchService(fs);
         return fs;
     }
@@ -213,11 +193,30 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
             ProviderNotFoundException, IOException, SecurityException {
         final FileSystem fs = super.newFileSystem(uri,
                                                   env);
-        index(fs);
+        if (shouldPerformInitialIndex(fs)) {
+            setupBatchIndex(fs);
+        }
         setupWatchService(fs);
         return fs;
     }
 
+    private boolean shouldPerformInitialIndex(FileSystem fs) {
+        return rootDirStream(fs).filter(dir -> hasContent(dir))
+                                .findAny()
+                                .isPresent() && indexEngine.freshIndex(KObjectUtil.toKCluster(fs));
+    }
+
+    private boolean hasContent(Path dir) {
+        // TODO remove this filter when AF-1073 is resolved
+        try (DirectoryStream<Path> children = newDirectoryStream(dir, path -> !path.endsWith("readme.md"))) {
+            return children.iterator().hasNext();
+        }
+    }
+
+    private Stream<Path> rootDirStream(FileSystem fs) {
+        return StreamSupport.stream(fs.getRootDirectories().spliterator(), false);
+    }
+
     @Override
     public int priority() {
         return 60;
@@ -231,6 +230,11 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
         super.dispose();
     }
 
+    private void setupBatchIndex(FileSystem fs) {
+        indexEngine.prepareBatch(KObjectUtil.toKCluster(fs));
+        batchIndex.runAsync(fs);
+    }
+
     protected void setupWatchService(final FileSystem fs) {
         if (watchedList.contains(fs.getName())) {
             return;
@@ -266,6 +270,131 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
 
                         @Override
                         public void run() {
+                            final KCluster kCluster = KObjectUtil.toKCluster(fs);
+                            final Set<Path> eventRealPaths = getRealCreatedPaths(events);
+                            try {
+                                indexEngine.startBatch(kCluster);
+                                processEvents(events, eventRealPaths);
+                                indexEngine.commit(kCluster);
+                            } catch (DisposedException e) {
+                                return;
+                            } finally {
+                                indexEngine.abort(kCluster);
+                            }
+                        }
+
+                        private void processEvents(final List<WatchEvent<?>> events, final Set<Path> eventRealPaths) throws DisposedException {
+                            for (WatchEvent event : events) {
+                                if (isDisposed()) {
+                                    throw new DisposedException();
+                                }
+                                try {
+                                    final WatchContext context = ((WatchContext) event.context());
+                                    processEvent(eventRealPaths, event, context);
+                                } catch (final Exception ex) {
+                                    LOGGER.error("Error during indexing. { " + event.toString() + " }",
+                                                 ex);
+                                }
+                            }
+                        }
+
+                        private void processEvent(final Set<Path> eventRealPaths, WatchEvent event, final WatchContext context) throws DisposedException {
+                            if (event.kind() == ENTRY_MODIFY || event.kind() == ENTRY_CREATE) {
+                                processCreationAndModificationEvent(eventRealPaths, context);
+                            }
+
+                            if (event.kind() == StandardWatchEventKind.ENTRY_RENAME) {
+                                processRenameEvent(context);
+                            }
+
+                            if (event.kind() == StandardWatchEventKind.ENTRY_DELETE) {
+                                processDeleteEvent(event, context);
+                            }
+                        }
+
+                        private void processDeleteEvent(WatchEvent object, final WatchContext context) throws DisposedException {
+                            //Default indexing
+                            final Path oldPath = context.getOldPath();
+                            indexEngine.delete(KObjectUtil.toKObjectKey(oldPath));
+
+                            //Additional indexing
+                            for (Indexer indexer : IndexersFactory.getIndexers()) {
+                                if (isDisposed()) {
+                                    throw new DisposedException();
+                                }
+                                if (indexer.supportsPath(oldPath)) {
+                                    final KObjectKey kObject = indexer.toKObjectKey(oldPath);
+                                    if (kObject != null) {
+                                        indexEngine.delete(kObject);
+                                    }
+                                }
+                            }
+                        }
+
+                        private void processRenameEvent(final WatchContext context) throws DisposedException {
+                            //Default indexing
+                            final Path sourcePath = context.getOldPath();
+                            final Path destinationPath = context.getPath();
+                            indexEngine.rename(KObjectUtil.toKObjectKey(sourcePath),
+                                               KObjectUtil.toKObject(destinationPath));
+
+                            //Additional indexing
+                            for (Indexer indexer : IndexersFactory.getIndexers()) {
+                                if (isDisposed()) {
+                                    throw new DisposedException();
+                                }
+                                if (indexer.supportsPath(destinationPath)) {
+                                    final KObjectKey kObjectSource = indexer.toKObjectKey(sourcePath);
+                                    final KObject kObjectDestination = indexer.toKObject(destinationPath);
+                                    if (kObjectSource != null && kObjectDestination != null) {
+                                        indexEngine.rename(kObjectSource,
+                                                           kObjectDestination);
+                                    }
+                                }
+                            }
+                        }
+
+                        private void processCreationAndModificationEvent(final Set<Path> eventRealPaths, final WatchContext context) throws DisposedException {
+                            // If the path to be indexed is a "dot path" but does not have an associated
+                            // "real path" index the "real path" instead. This ensures when only a
+                            // "dot path" is updated the FileAttributeView(s) are re-indexed.
+                            Path path = context.getPath();
+                            if (path.getFileName().toString().startsWith(".")) {
+                                if (!IOServiceIndexedUtil.isBlackListed(path)) {
+                                    final Path realPath = DotFileUtils.undot(path);
+                                    if (!eventRealPaths.contains(realPath)) {
+                                        path = realPath;
+                                    }
+                                }
+                            }
+
+                            if (!path.getFileName().toString().startsWith(".")) {
+
+                                //Default indexing
+                                for (final Class<? extends FileAttributeView> view : views) {
+                                    getFileAttributeView(path,
+                                                         view);
+                                }
+                                final FileAttribute<?>[] allAttrs = convert(readAttributes(path));
+                                indexEngine.index(KObjectUtil.toKObject(path,
+                                                                        allAttrs));
+
+                                //Additional indexing
+                                for (Indexer indexer : IndexersFactory.getIndexers()) {
+                                    if (isDisposed()) {
+                                        throw new DisposedException();
+                                    }
+                                    if (indexer.supportsPath(path)) {
+                                        final KObject kObject = indexer.toKObject(path);
+                                        if (kObject != null) {
+                                            indexEngine.index(kObject);
+                                        }
+                                    }
+                                }
+                            }
+                        }
+
+                        private Set<Path> getRealCreatedPaths(final List<WatchEvent<?>> events) {
                             // Get a set of "real paths" to be indexed. The "dot path" associated with the "real path"
                             // is automatically indexed because the "dot path" contains content for FileAttributeView(s)
                             // linked to the "real path".
@@ -279,100 +408,7 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
                                     }
                                 }
                             }
-
-                            for (WatchEvent object : events) {
-                                if (isDisposed()) {
-                                    return;
-                                }
-                                try {
-                                    final WatchContext context = ((WatchContext) object.context());
-                                    if (object.kind() == ENTRY_MODIFY || object.kind() == ENTRY_CREATE) {
-
-                                        // If the path to be indexed is a "dot path" but does not have an associated
-                                        // "real path" index the "real path" instead. This ensures when only a
-                                        // "dot path" is updated the FileAttributeView(s) are re-indexed.
-                                        Path path = context.getPath();
-                                        if (path.getFileName().toString().startsWith(".")) {
-                                            if (!IOServiceIndexedUtil.isBlackListed(path)) {
-                                                final Path realPath = DotFileUtils.undot(path);
-                                                if (!eventRealPaths.contains(realPath)) {
-                                                    path = realPath;
-                                                }
-                                            }
-                                        }
-
-                                        if (!path.getFileName().toString().startsWith(".")) {
-
-                                            //Default indexing
-                                            for (final Class<? extends FileAttributeView> view : views) {
-                                                getFileAttributeView(path,
-                                                                     view);
-                                            }
-                                            final FileAttribute<?>[] allAttrs = convert(readAttributes(path));
-                                            indexEngine.index(KObjectUtil.toKObject(path,
-                                                                                    allAttrs));
-
-                                            //Additional indexing
-                                            for (Indexer indexer : IndexersFactory.getIndexers()) {
-                                                if (isDisposed()) {
-                                                    return;
-                                                }
-                                                if (indexer.supportsPath(path)) {
-                                                    final KObject kObject = indexer.toKObject(path);
-                                                    if (kObject != null) {
-                                                        indexEngine.index(kObject);
-                                                    }
-                                                }
-                                            }
-                                        }
-                                    }
-
-                                    if (object.kind() == StandardWatchEventKind.ENTRY_RENAME) {
-                                        //Default indexing
-                                        final Path sourcePath = context.getOldPath();
-                                        final Path destinationPath = context.getPath();
-                                        indexEngine.rename(KObjectUtil.toKObjectKey(sourcePath),
-                                                           KObjectUtil.toKObject(destinationPath));
-
-                                        //Additional indexing
-                                        for (Indexer indexer : IndexersFactory.getIndexers()) {
-                                            if (isDisposed()) {
-                                                return;
-                                            }
-                                            if (indexer.supportsPath(destinationPath)) {
-                                                final KObjectKey kObjectSource = indexer.toKObjectKey(sourcePath);
-                                                final KObject kObjectDestination = indexer.toKObject(destinationPath);
-                                                if (kObjectSource != null && kObjectDestination != null) {
-                                                    indexEngine.rename(kObjectSource,
-                                                                       kObjectDestination);
-                                                }
-                                            }
-                                        }
-                                    }
-
-                                    if (object.kind() == StandardWatchEventKind.ENTRY_DELETE) {
-                                        //Default indexing
-                                        final Path oldPath = context.getOldPath();
-                                        indexEngine.delete(KObjectUtil.toKObjectKey(oldPath));
-
-                                        //Additional indexing
-                                        for (Indexer indexer : IndexersFactory.getIndexers()) {
-                                            if (isDisposed()) {
-                                                return;
-                                            }
-                                            if (indexer.supportsPath(oldPath)) {
-                                                final KObjectKey kObject = indexer.toKObjectKey(oldPath);
-                                                if (kObject != null) {
-                                                    indexEngine.delete(kObject);
-                                                }
-                                            }
-                                        }
-                                    }
-                                } catch (final Exception ex) {
-                                    LOGGER.error("Error during indexing. { " + object.toString() + " }",
-                                                 ex);
-                                }
-                            }
+                            return eventRealPaths;
                         }
 
                         private boolean isDisposed() {
@@ -385,21 +421,6 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
         });
     }
 
-    private synchronized void indexIfFresh(final FileSystem fs) {
-        final KCluster cluster = KObjectUtil.toKCluster(fs);
-        if (indexEngine.freshIndex(cluster)) {
-            // See https://bugzilla.redhat.com/show_bug.cgi?id=1288132
-            // Record batch index as being started before the async indexing actually runs to
-            // prevent multiple batch indexes for the same FileSystem being scheduled.
-            indexEngine.startBatch(cluster);
-            index(fs);
-        }
-    }
-
-    private void index(final FileSystem fs) {
-        batchIndex.runAsync(fs);
-    }
-
     @Override
     public void delete(final Path path,
                        final DeleteOption... options) throws IllegalArgumentException, NoSuchFileException, DirectoryNotEmptyException, IOException, SecurityException {
@@ -427,6 +448,10 @@ public class IOServiceIndexedImpl extends IOServiceDotFileImpl {
         return indexEngine;
     }
 
+    private static class DisposedException extends Exception {
+        private static final long serialVersionUID = 1L;
+    }
+
     /**
      * A "No Operation" Observer, used by default
      */
