@@ -15,13 +15,7 @@
  */
 package org.uberfire.client.mvp;
 
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collection;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
+import java.util.*;
 import java.util.function.Predicate;
 import java.util.function.Supplier;
 
@@ -98,7 +92,8 @@ public class PlaceManagerImpl
      */
     private final Map<PlaceRequest, CustomPanelDefinition> customPanels = new HashMap<>();
 
-    private final Map<PlaceRequest, Command> onOpenCallbacks = new HashMap<PlaceRequest, Command>();
+    private final Map<PlaceRequest, List<Command>> onOpenCallbacks = new HashMap<>();
+    private final Map<PlaceRequest, List<Command>> onCloseCallbacks = new HashMap<>();
     /**
      * Splash screens that have intercepted some other activity which is currently part of the workbench. Each of these
      * splash screens may or may not be visible (they manage their own "show next time" preferences).
@@ -222,7 +217,7 @@ public class PlaceManagerImpl
     }
 
     private void closeOpenPlacesAt(Predicate<CustomPanelDefinition> filterPanels) {
-        customPanels.values().stream()
+        new HashSet<>(customPanels.values()).stream()
                 .filter(filterPanels)
                 .flatMap(p -> p.getParts().stream())
                 .forEach(part -> closePlace(part.getPlace()));
@@ -305,7 +300,7 @@ public class PlaceManagerImpl
         boolean result = true;
         for (final PlaceRequest placeRequest : placeRequests) {
             final Activity activity = existingWorkbenchActivities.get(placeRequest);
-            if (activity.isType(ActivityResourceType.SCREEN.name()) || activity.isType(ActivityResourceType.EDITOR.name())) {
+            if (activity != null && (activity.isType(ActivityResourceType.SCREEN.name()) || activity.isType(ActivityResourceType.EDITOR.name()))) {
                 if (((WorkbenchActivity) activity).onMayClose()) {
                     onMayCloseList.put(placeRequest,
                                        activity);
@@ -574,22 +569,54 @@ public class PlaceManagerImpl
 
     @Override
     public void registerOnOpenCallback(final PlaceRequest place,
-                                       final Command command) {
+                                       final Command callback) {
         checkNotNull("place",
                      place);
-        checkNotNull("command",
-                     command);
-        this.onOpenCallbacks.put(place,
-                                 command);
+        checkNotNull("callback",
+                     callback);
+
+        List<Command> callbacks = getOnOpenCallbacks(place);
+        if (callbacks == null) {
+            callbacks = new ArrayList<>();
+            this.onOpenCallbacks.put(place,
+                                     callbacks);
+        }
+
+        callbacks.add(callback);
     }
 
     @Override
-    public void unregisterOnOpenCallback(final PlaceRequest place) {
+    public void unregisterOnOpenCallbacks(final PlaceRequest place) {
         checkNotNull("place",
                      place);
         this.onOpenCallbacks.remove(place);
     }
 
+    @Override
+    public void registerOnCloseCallback(final PlaceRequest place,
+                                        final Command callback) {
+        checkNotNull("place",
+                     place);
+        checkNotNull("callback",
+                     callback);
+
+        List<Command> callbacks = getOnCloseCallbacks(place);
+        if (callbacks == null) {
+            callbacks = new ArrayList<>();
+            this.onCloseCallbacks.put(place,
+                                     callbacks);
+        }
+
+        callbacks.add(callback);
+    }
+
+    @Override
+    public void unregisterOnCloseCallbacks(final PlaceRequest place) {
+        checkNotNull("place",
+                     place);
+        this.onCloseCallbacks.remove(place);
+    }
+
     /**
      * Finds and opens the splash screen for the given place, if such a splash screen exists. The splash screen might
      * not actually display; each splash screen keeps track of its own preference setting for whether or not the user
@@ -1050,10 +1077,15 @@ public class PlaceManagerImpl
     }
 
     @Override
-    public Command getOpenCallback(PlaceRequest place) {
+    public List<Command> getOnOpenCallbacks(final PlaceRequest place) {
         return this.onOpenCallbacks.get(place);
     }
 
+    @Override
+    public List<Command> getOnCloseCallbacks(final PlaceRequest place) {
+        return this.onCloseCallbacks.get(place);
+    }
+
     /**
      * The result of an attempt to resolve a PlaceRequest to an Activity.
      */
